@@ -1,4 +1,4 @@
-{-# LANGUAGE CPP #-}
+{-# LANGUAGE CPP, TupleSections #-}
 {-
 Copyright (C) 2006-2014 John MacFarlane <jgm@berkeley.edu>
 
@@ -33,10 +33,12 @@ module Main where
 import Text.Pandoc
 import Text.Pandoc.Builder (setMeta)
 import Text.Pandoc.PDF (makePDF)
+import Text.Pandoc.Walk (walk)
 import Text.Pandoc.Readers.LaTeX (handleIncludes)
 import Text.Pandoc.Shared ( tabFilter, readDataFileUTF8, readDataFile,
                             safeRead, headerShift, normalize, err, warn,
                             openURL )
+import Text.Pandoc.MediaBag ( mediaDirectory, extractMediaBag, MediaBag )
 import Text.Pandoc.XML ( toEntities )
 import Text.Pandoc.SelfContained ( makeSelfContained )
 import Text.Pandoc.Process (pipeProcess)
@@ -49,30 +51,37 @@ import System.Console.GetOpt
 import Data.Char ( toLower )
 import Data.List ( intercalate, isPrefixOf, isSuffixOf, sort )
 import System.Directory ( getAppUserDataDirectory, findExecutable,
-                          doesFileExist )
+                          doesFileExist, Permissions(..), getPermissions )
 import System.IO ( stdout, stderr )
 import System.IO.Error ( isDoesNotExistError )
 import qualified Control.Exception as E
 import Control.Exception.Extensible ( throwIO )
 import qualified Text.Pandoc.UTF8 as UTF8
-import Control.Monad (when, unless, liftM)
+import Control.Monad (when, unless, (>=>))
+import Data.Maybe (isJust, fromMaybe)
 import Data.Foldable (foldrM)
 import Network.URI (parseURI, isURI, URI(..))
 import qualified Data.ByteString.Lazy as B
 import qualified Data.ByteString as BS
 import Data.Aeson (eitherDecode', encode)
 import qualified Data.Map as M
-import System.IO.Error(ioeGetErrorType)
-import GHC.IO.Exception (IOErrorType(ResourceVanished))
 import Data.Yaml (decode)
 import qualified Data.Yaml as Yaml
 import qualified Data.Text as T
+import Control.Applicative ((<$>), (<|>))
+import Text.Pandoc.Readers.Txt2Tags (getT2TMeta)
+import Data.Monoid
+
+type Transform = Pandoc -> Pandoc
 
 copyrightMessage :: String
-copyrightMessage = "\nCopyright (C) 2006-2014 John MacFarlane\n" ++
-                    "Web:  http://johnmacfarlane.net/pandoc\n" ++
-                    "This is free software; see the source for copying conditions.  There is no\n" ++
-                    "warranty, not even for merchantability or fitness for a particular purpose."
+copyrightMessage = intercalate "\n" [
+  "",
+  "Copyright (C) 2006-2014 John MacFarlane",
+  "Web:  http://johnmacfarlane.net/pandoc",
+  "This is free software; see the source for copying conditions.",
+  "There is no warranty, not even for merchantability or fitness",
+  "for a particular purpose." ]
 
 compileInfo :: String
 compileInfo =
@@ -86,26 +95,44 @@ compileInfo =
 -- comma separated words in lines with a maximum line length.
 wrapWords :: Int -> Int -> [String] -> String
 wrapWords indent c = wrap' (c - indent) (c - indent)
-        where wrap' _    _         []     = ""
-              wrap' cols remaining (x:xs) = if remaining == cols
-                                               then x ++ wrap' cols (remaining - length x) xs
-                                               else if (length x + 1) > remaining
-                                                       then ",\n" ++ replicate indent ' ' ++ x ++ wrap' cols (cols - length x) xs
-                                                       else ", "  ++ x ++ wrap' cols (remaining - (length x + 2)) xs
+  where
+    wrap'   _       _      []   = ""
+    wrap' cols remaining (x:xs)
+      | remaining == cols =
+            x ++ wrap' cols (remaining - length x) xs
+      | (length x + 1) > remaining =
+            ",\n" ++ replicate indent ' ' ++ x ++
+            wrap' cols (cols - length x) xs
+      | otherwise =
+            ", "  ++ x ++
+            wrap' cols (remaining - length x - 2) xs
 
 isTextFormat :: String -> Bool
-isTextFormat s = takeWhile (`notElem` "+-") s `notElem` ["odt","docx","epub","epub3"]
+isTextFormat s = takeWhile (`notElem` "+-") s `notElem` binaries
+  where binaries = ["odt","docx","epub","epub3"]
 
 externalFilter :: FilePath -> [String] -> Pandoc -> IO Pandoc
 externalFilter f args' d = do
-      mbexe <- findExecutable f
+      mbexe <- if '/' `elem` f
+                  -- don't check PATH if filter name has a path
+                  then return Nothing
+                  -- we catch isDoesNotExistError because this will
+                  -- be triggered if PATH not set:
+                  else E.catch (findExecutable f)
+                                 (\e -> if isDoesNotExistError e
+                                           then return Nothing
+                                           else throwIO e)
       (f', args'') <- case mbexe of
                            Just x  -> return (x, args')
                            Nothing -> do
                              exists <- doesFileExist f
                              if exists
-                                then return $
+                                then do
+                                  isExecutable <- executable `fmap`
+                                                    getPermissions f
+                                  return $
                                     case map toLower $ takeExtension f of
+                                         _ | isExecutable -> (f, args')
                                          ".py"  -> ("python", f:args')
                                          ".hs"  -> ("runhaskell", f:args')
                                          ".pl"  -> ("perl", f:args')
@@ -121,10 +148,7 @@ externalFilter f args' d = do
            ExitFailure _  -> err 83 $ "Error running filter " ++ f
  where filterException :: E.SomeException -> IO a
        filterException e = err 83 $ "Error running filter " ++ f ++ "\n" ++
-                                  if ioeGetErrorType `fmap` E.fromException e ==
-                                          Just ResourceVanished
-                                     then f ++ " not found in path"
-                                     else show e
+                                       show e
 
 -- | Data structure for command line options.
 data Opt = Opt
@@ -135,7 +159,7 @@ data Opt = Opt
     , optWriter            :: String  -- ^ Writer format
     , optParseRaw          :: Bool    -- ^ Parse unconvertable HTML and TeX
     , optTableOfContents   :: Bool    -- ^ Include table of contents
-    , optTransforms        :: [Pandoc -> Pandoc]  -- ^ Doc transforms to apply
+    , optTransforms        :: [Transform]  -- ^ Doc transforms to apply
     , optTemplate          :: Maybe FilePath  -- ^ Custom template
     , optVariables         :: [(String,String)] -- ^ Template variables to set
     , optMetadata          :: M.Map String MetaValue -- ^ Metadata fields to set
@@ -178,7 +202,11 @@ data Opt = Opt
     , optAscii             :: Bool       -- ^ Use ascii characters only in html
     , optTeXLigatures      :: Bool       -- ^ Use TeX ligatures for quotes/dashes
     , optDefaultImageExtension :: String -- ^ Default image extension
+    , optExtractMedia      :: Maybe FilePath -- ^ Path to extract embedded media
     , optTrace             :: Bool       -- ^ Print debug information
+    , optTrackChanges      :: TrackChanges -- ^ Accept or reject MS Word track-changes.
+    , optKaTeXStylesheet   :: Maybe String     -- ^ Path to stylesheet for KaTeX
+    , optKaTeXJS           :: Maybe String     -- ^ Path to js file for KaTeX
     }
 
 -- | Defaults for command-line options.
@@ -234,7 +262,11 @@ defaultOpts = Opt
     , optAscii                 = False
     , optTeXLigatures          = True
     , optDefaultImageExtension = ""
+    , optExtractMedia          = Nothing
     , optTrace                 = False
+    , optTrackChanges          = AcceptChanges
+    , optKaTeXStylesheet       = Nothing
+    , optKaTeXJS               = Nothing
     }
 
 -- | A list of functions, each transforming the options data structure
@@ -337,6 +369,26 @@ options =
                   "NUMBER")
                  "" -- "Tab stop (default 4)"
 
+    , Option "" ["track-changes"]
+                 (ReqArg
+                  (\arg opt -> do
+                     action <- case arg of
+                            "accept" -> return AcceptChanges
+                            "reject" -> return RejectChanges
+                            "all"    -> return AllChanges
+                            _        -> err 6
+                               ("Unknown option for track-changes: " ++ arg)
+                     return opt { optTrackChanges = action })
+                  "accept|reject|all")
+                 "" -- "Accepting or reject MS Word track-changes.""
+
+    , Option "" ["extract-media"]
+                 (ReqArg
+                  (\arg opt -> do
+                    return opt { optExtractMedia = Just arg })
+                  "PATH")
+                 "" -- "Directory to which to extract embedded media"
+
     , Option "s" ["standalone"]
                  (NoArg
                   (\opt -> return opt { optStandalone = True }))
@@ -766,10 +818,25 @@ options =
                   (\arg opt -> do
                       let url' = case arg of
                                       Just u   -> u
-                                      Nothing  -> "http://cdn.mathjax.org/mathjax/latest/MathJax.js?config=TeX-AMS-MML_HTMLorMML"
+                                      Nothing  -> "//cdn.mathjax.org/mathjax/latest/MathJax.js?config=TeX-AMS-MML_HTMLorMML"
                       return opt { optHTMLMathMethod = MathJax url'})
                   "URL")
                  "" -- "Use MathJax for HTML math"
+    , Option "" ["katex"]
+                 (OptArg
+                  (\arg opt ->
+                      return opt
+                        { optKaTeXJS =
+                           arg <|> Just "http://cdnjs.cloudflare.com/ajax/libs/KaTeX/0.1.0/katex.min.js"})
+                  "URL")
+                  "" -- Use KaTeX for HTML Math
+
+    , Option "" ["katex-stylesheet"]
+                 (ReqArg
+                  (\arg opt ->
+                      return opt { optKaTeXStylesheet = Just arg })
+                 "URL")
+                 "" -- Set the KaTeX Stylesheet location
 
     , Option "" ["gladtex"]
                  (NoArg
@@ -812,6 +879,7 @@ options =
 
     ]
 
+
 addMetadata :: String -> MetaValue -> M.Map String MetaValue
             -> M.Map String MetaValue
 addMetadata k v m = case M.lookup k m of
@@ -855,9 +923,16 @@ defaultReaderName fallback (x:xs) =
     ".db"       -> "docbook"
     ".opml"     -> "opml"
     ".wiki"     -> "mediawiki"
+    ".dokuwiki" -> "dokuwiki"
     ".textile"  -> "textile"
     ".native"   -> "native"
     ".json"     -> "json"
+    ".docx"     -> "docx"
+    ".t2t"      -> "t2t"
+    ".epub"     -> "epub"
+    ".odt"      -> "odt"  -- so we get an "unknown reader" error
+    ".pdf"      -> "pdf"  -- so we get an "unknown reader" error
+    ".doc"      -> "doc"  -- so we get an "unknown reader" error
     _           -> defaultReaderName fallback xs
 
 -- Returns True if extension of first source is .lhs
@@ -898,13 +973,39 @@ defaultWriterName x =
     ".pdf"      -> "latex"
     ".fb2"      -> "fb2"
     ".opml"     -> "opml"
+    ".icml"     -> "icml"
     ['.',y] | y `elem` ['1'..'9'] -> "man"
-    _          -> "html"
+    _           -> "html"
+
+-- Transformations of a Pandoc document post-parsing:
+
+extractMedia :: MediaBag -> FilePath -> Pandoc -> IO Pandoc
+extractMedia media dir d =
+  case [fp | (fp, _, _) <- mediaDirectory media] of
+        []  -> return d
+        fps -> do
+          extractMediaBag True dir media
+          return $ walk (adjustImagePath dir fps) d
+
+adjustImagePath :: FilePath -> [FilePath] -> Inline -> Inline
+adjustImagePath dir paths (Image lab (src, tit))
+   | src `elem` paths = Image lab (dir ++ "/" ++ src, tit)
+adjustImagePath _ _ x = x
+
+adjustMetadata :: M.Map String MetaValue -> Pandoc -> IO Pandoc
+adjustMetadata metadata d = return $ M.foldWithKey setMeta d metadata
+
+applyTransforms :: [Transform] -> Pandoc -> IO Pandoc
+applyTransforms transforms d = return $ foldr ($) d transforms
+
+applyFilters :: [FilePath] -> [String] -> Pandoc -> IO Pandoc
+applyFilters filters args d =
+  foldrM ($) d $ map (flip externalFilter args) filters
 
 main :: IO ()
 main = do
 
-  rawArgs <- liftM (map UTF8.decodeArg) getArgs
+  rawArgs <- map UTF8.decodeArg <$> getArgs
   prg <- getProgName
   let compatMode = (prg == "hsmarkdown")
 
@@ -939,7 +1040,7 @@ main = do
               , optTemplate              = templatePath
               , optOutputFile            = outputFile
               , optNumberSections        = numberSections
-              , optNumberOffset            = numberFrom
+              , optNumberOffset          = numberFrom
               , optSectionDivs           = sectionDivs
               , optIncremental           = incremental
               , optSelfContained         = selfContained
@@ -950,7 +1051,7 @@ main = do
               , optHighlight             = highlight
               , optHighlightStyle        = highlightStyle
               , optChapters              = chapters
-              , optHTMLMathMethod        = mathMethod
+              , optHTMLMathMethod        = mathMethod'
               , optReferenceODT          = referenceODT
               , optReferenceDocx         = referenceDocx
               , optEpubStylesheet        = epubStylesheet
@@ -976,7 +1077,11 @@ main = do
               , optAscii                 = ascii
               , optTeXLigatures          = texLigatures
               , optDefaultImageExtension = defaultImageExtension
+              , optExtractMedia          = mbExtractMedia
               , optTrace                 = trace
+              , optTrackChanges          = trackChanges
+              , optKaTeXStylesheet       = katexStylesheet
+              , optKaTeXJS               = katexJS
              } = opts
 
   when dumpArgs $
@@ -984,20 +1089,25 @@ main = do
        mapM_ (\arg -> UTF8.hPutStrLn stdout arg) args
        exitWith ExitSuccess
 
+  let csscdn = "http://cdnjs.cloudflare.com/ajax/libs/KaTeX/0.1.0/katex.min.css"
+  let mathMethod =
+        case (katexJS, katexStylesheet) of
+            (Nothing, _) -> mathMethod'
+            (Just js, ss) -> KaTeX js (fromMaybe csscdn ss)
+
+
   -- --bibliography implies -F pandoc-citeproc for backwards compatibility:
-  let filters' = case M.lookup "bibliography" metadata of
-                       Just _ | optCiteMethod opts /= Natbib &&
-                                optCiteMethod opts /= Biblatex &&
-                                all (\f -> takeBaseName f /= "pandoc-citeproc")
-                                filters -> "pandoc-citeproc" : filters
-                       _                -> filters
-  let plugins = map externalFilter filters'
+  let needsCiteproc = isJust (M.lookup "bibliography" metadata) &&
+                      optCiteMethod opts `notElem` [Natbib, Biblatex] &&
+                      "pandoc-citeproc" `notElem` map takeBaseName filters
+  let filters' = if needsCiteproc then "pandoc-citeproc" : filters
+                                  else filters
 
   let sources = if ignoreArgs then [] else args
 
   datadir <- case mbDataDir of
                   Nothing   -> E.catch
-                                 (liftM Just $ getAppUserDataDirectory "pandoc")
+                                 (Just <$> getAppUserDataDirectory "pandoc")
                                  (\e -> let _ = (e :: E.SomeException)
                                         in  return Nothing)
                   Just _    -> return mbDataDir
@@ -1028,16 +1138,29 @@ main = do
                else case getWriter writerName' of
                          Left e  -> err 9 $
                            if writerName' == "pdf"
-                              then e ++ "\nTo create a pdf with pandoc, use " ++
+                              then e ++
+                               "\nTo create a pdf with pandoc, use " ++
                                "the latex or beamer writer and specify\n" ++
                                "an output file with .pdf extension " ++
                                "(pandoc -t latex -o filename.pdf)."
                               else e
                          Right w -> return w
 
-  reader <- case getReader readerName' of
-     Right r  -> return r
-     Left e   -> err 7 e
+  reader <- if "t2t" == readerName'
+              then (mkStringReader .
+                    readTxt2Tags) <$>
+                      (getT2TMeta sources outputFile)
+              else case getReader readerName' of
+                Right r  -> return r
+                Left e   -> err 7 e'
+                  where e' = case readerName' of
+                                  "odt" -> e ++
+                                    "\nPandoc can convert to ODT, but not from ODT.\nTry using LibreOffice to export as HTML, and convert that with pandoc."
+                                  "pdf" -> e ++
+                                     "\nPandoc can convert to PDF, but not from PDF."
+                                  "doc" -> e ++
+                                     "\nPandoc can convert from DOCX, but not from DOC.\nTry using Word to save your DOC file as DOCX, and convert that with pandoc."
+                                  _ -> e
 
   let standalone' = standalone || not (isTextFormat writerName') || pdfOutput
 
@@ -1076,19 +1199,21 @@ main = do
                     then do
                         dztempl <- readDataFileUTF8 datadir
                                      ("dzslides" </> "template.html")
-                        let dzcore = unlines $ dropWhile (not . isPrefixOf "<!-- {{{{ dzslides core")
-                                             $ lines dztempl
+                        let dzline = "<!-- {{{{ dzslides core"
+                        let dzcore = unlines
+                                   $ dropWhile (not . (dzline `isPrefixOf`))
+                                   $ lines dztempl
                         return $ ("dzslides-core", dzcore) : variables'
                     else return variables'
+
   let sourceURL = case sources of
-                        []    -> Nothing
-                        (x:_) -> case parseURI x of
-                                    Just u
-                                      | uriScheme u `elem` ["http:","https:"] ->
-                                          Just $ show u{ uriPath = "",
-                                                         uriQuery = "",
-                                                         uriFragment = "" }
-                                    _ -> Nothing
+                    []    -> Nothing
+                    (x:_) -> case parseURI x of
+                                Just u
+                                  | uriScheme u `elem` ["http:","https:"] ->
+                                      Just $ show u{ uriQuery = "",
+                                                     uriFragment = "" }
+                                _ -> Nothing
 
   let readerOpts = def{ readerSmart = smart || (texLigatures &&
                           (laTeXOutput || "context" `isPrefixOf` writerName'))
@@ -1101,8 +1226,46 @@ main = do
                       , readerApplyMacros = not laTeXOutput
                       , readerDefaultImageExtension = defaultImageExtension
                       , readerTrace = trace
+                      , readerTrackChanges = trackChanges
                       }
 
+  when (not (isTextFormat writerName') && outputFile == "-") $
+    err 5 $ "Cannot write " ++ writerName' ++ " output to stdout.\n" ++
+            "Specify an output file using the -o option."
+
+  let readSources [] = mapM readSource ["-"]
+      readSources srcs = mapM readSource srcs
+      readSource "-" = UTF8.getContents
+      readSource src = case parseURI src of
+                            Just u | uriScheme u `elem` ["http:","https:"] ->
+                                       readURI src
+                            _       -> UTF8.readFile src
+      readURI src = do
+        res <- openURL src
+        case res of
+             Left e        -> throwIO e
+             Right (bs,_)  -> return $ UTF8.toString bs
+
+  let readFiles [] = error "Cannot read archive from stdin"
+      readFiles (x:_) = B.readFile x
+
+  let convertTabs = tabFilter (if preserveTabs || readerName' == "t2t"
+                                 then 0
+                                 else tabStop)
+
+  let handleIncludes' = if readerName' == "latex" ||
+                           readerName' == "latex+lhs"
+                               then handleIncludes
+                               else return
+
+  (doc, media) <-
+     case reader of
+          StringReader r-> (, mempty) <$>
+            (  readSources >=>
+               handleIncludes' . convertTabs . intercalate "\n" >=>
+               r readerOpts ) sources
+          ByteStringReader r -> readFiles sources >>= r readerOpts
+
   let writerOptions = def { writerStandalone       = standalone',
                             writerTemplate         = templ,
                             writerVariables        = variables'',
@@ -1138,40 +1301,15 @@ main = do
                             writerEpubChapterLevel = epubChapterLevel,
                             writerTOCDepth         = epubTOCDepth,
                             writerReferenceODT     = referenceODT,
-                            writerReferenceDocx    = referenceDocx
+                            writerReferenceDocx    = referenceDocx,
+                            writerMediaBag         = media
                           }
 
-  when (not (isTextFormat writerName') && outputFile == "-") $
-    err 5 $ "Cannot write " ++ writerName' ++ " output to stdout.\n" ++
-            "Specify an output file using the -o option."
 
-  let readSources [] = mapM readSource ["-"]
-      readSources srcs = mapM readSource srcs
-      readSource "-" = UTF8.getContents
-      readSource src = case parseURI src of
-                            Just u | uriScheme u `elem` ["http:","https:"] ->
-                                       readURI src
-                            _       -> UTF8.readFile src
-      readURI src = do
-        res <- openURL src
-        case res of
-             Left e        -> throwIO e
-             Right (bs,_)  -> return $ UTF8.toString bs
-
-  let convertTabs = tabFilter (if preserveTabs then 0 else tabStop)
-
-  let handleIncludes' = if readerName' == "latex" || readerName' == "latex+lhs"
-                           then handleIncludes
-                           else return
-
-  doc <- readSources sources >>=
-           handleIncludes' . convertTabs . intercalate "\n" >>=
-           reader readerOpts
-
-
-  let doc0 = M.foldWithKey setMeta doc metadata
-  let doc1 = foldr ($) doc0 transforms
-  doc2 <- foldrM ($) doc1 $ map ($ [writerName']) plugins
+  doc' <- (maybe return (extractMedia media) mbExtractMedia >=>
+           adjustMetadata metadata >=>
+           applyTransforms transforms >=>
+           applyFilters filters' [writerName']) doc
 
   let writeBinary :: B.ByteString -> IO ()
       writeBinary = B.writeFile (UTF8.encodePath outputFile)
@@ -1181,8 +1319,8 @@ main = do
       writerFn f   = UTF8.writeFile f
 
   case writer of
-    IOStringWriter f -> f writerOptions doc2 >>= writerFn outputFile
-    IOByteStringWriter f -> f writerOptions doc2 >>= writeBinary
+    IOStringWriter f -> f writerOptions doc' >>= writerFn outputFile
+    IOByteStringWriter f -> f writerOptions doc' >>= writeBinary
     PureStringWriter f
       | pdfOutput -> do
               -- make sure writer is latex or beamer
@@ -1196,21 +1334,21 @@ main = do
                    err 41 $ latexEngine ++ " not found. " ++
                      latexEngine ++ " is needed for pdf output."
 
-              res <- makePDF latexEngine f writerOptions doc2
+              res <- makePDF latexEngine f writerOptions doc'
               case res of
                    Right pdf -> writeBinary pdf
                    Left err' -> do
                      B.hPutStr stderr $ err'
                      B.hPut stderr $ B.pack [10]
                      err 43 "Error producing PDF from TeX source"
-      | otherwise -> selfcontain (f writerOptions doc2 ++
+      | otherwise -> selfcontain (f writerOptions doc' ++
                                   ['\n' | not standalone'])
                       >>= writerFn outputFile . handleEntities
           where htmlFormat = writerName' `elem`
                                ["html","html+lhs","html5","html5+lhs",
                                "s5","slidy","slideous","dzslides","revealjs"]
                 selfcontain = if selfContained && htmlFormat
-                                 then makeSelfContained datadir
+                                 then makeSelfContained writerOptions
                                  else return
                 handleEntities = if htmlFormat && ascii
                                     then toEntities
