@@ -31,6 +31,7 @@ Conversion of LaTeX to 'Pandoc' document.
 module Text.Pandoc.Readers.LaTeX ( readLaTeX,
                                    rawLaTeXInline,
                                    rawLaTeXBlock,
+                                   inlineCommand,
                                    handleIncludes
                                  ) where
 
@@ -40,7 +41,6 @@ import Text.Pandoc.Shared
 import Text.Pandoc.Options
 import Text.Pandoc.Parsing hiding ((<|>), many, optional, space,
                                    mathDisplay, mathInline)
-import Text.Parsec.Prim (ParsecT, runParserT)
 import qualified Text.Pandoc.UTF8 as UTF8
 import Data.Char ( chr, ord )
 import Control.Monad.Trans (lift)
@@ -103,7 +103,7 @@ dimenarg = try $ do
 
 sp :: LP ()
 sp = skipMany1 $ satisfy (\c -> c == ' ' || c == '\t')
-        <|> (try $ newline >>~ lookAhead anyChar >>~ notFollowedBy blankline)
+        <|> (try $ newline <* lookAhead anyChar <* notFollowedBy blankline)
 
 isLowerHex :: Char -> Bool
 isLowerHex x = x >= '0' && x <= '9' || x >= 'a' && x <= 'f'
@@ -191,6 +191,7 @@ inline = (mempty <$ comment)
      <|> (space  <$ sp)
      <|> inlineText
      <|> inlineCommand
+     <|> inlineEnvironment
      <|> inlineGroup
      <|> (char '-' *> option (str "-")
            ((char '-') *> option (str "–") (str "—" <$ char '-')))
@@ -304,7 +305,7 @@ blockCommands = M.fromList $
   , ("item", skipopts *> loose_item)
   , ("documentclass", skipopts *> braced *> preamble)
   , ("centerline", (para . trimInlines) <$> (skipopts *> tok))
-  , ("caption", tok >>= setCaption)
+  , ("caption", skipopts *> setCaption)
   , ("PandocStartInclude", startInclude)
   , ("PandocEndInclude", endInclude)
   , ("bibliography", mempty <$ (skipopts *> braced >>=
@@ -336,9 +337,16 @@ addMeta field val = updateState $ \st ->
 splitBibs :: String -> [Inlines]
 splitBibs = map (str . flip replaceExtension "bib" . trim) . splitBy (==',')
 
-setCaption :: Inlines -> LP Blocks
-setCaption ils = do
-  updateState $ \st -> st{ stateCaption = Just ils }
+setCaption :: LP Blocks
+setCaption = do
+  ils <- tok
+  mblabel <- option Nothing $
+               try $ spaces >> controlSeq "label" >> (Just <$> tok)
+  let ils' = case mblabel of
+                  Just lab -> ils <> spanWith
+                                ("",[],[("data-label", stringify lab)]) mempty
+                  Nothing  -> ils
+  updateState $ \st -> st{ stateCaption = Just ils' }
   return mempty
 
 resetCaption :: LP ()
@@ -394,29 +402,52 @@ unlessParseRaw = getOption readerParseRaw >>= guard . not
 isBlockCommand :: String -> Bool
 isBlockCommand s = maybe False (const True) $ M.lookup s blockCommands
 
+
+inlineEnvironments :: M.Map String (LP Inlines)
+inlineEnvironments = M.fromList
+  [ ("displaymath", mathEnv id Nothing "displaymath")
+  , ("equation", mathEnv id Nothing "equation")
+  , ("equation*", mathEnv id Nothing "equation*")
+  , ("gather", mathEnv id (Just "gathered") "gather")
+  , ("gather*", mathEnv id (Just "gathered") "gather*")
+  , ("multline", mathEnv id (Just "gathered") "multline")
+  , ("multline*", mathEnv id (Just "gathered") "multline*")
+  , ("eqnarray", mathEnv id (Just "aligned") "eqnarray")
+  , ("eqnarray*", mathEnv id (Just "aligned") "eqnarray*")
+  , ("align", mathEnv id (Just "aligned") "align")
+  , ("align*", mathEnv id (Just "aligned") "align*")
+  , ("alignat", mathEnv id (Just "aligned") "alignat")
+  , ("alignat*", mathEnv id (Just "aligned") "alignat*")
+  ]
+
 inlineCommands :: M.Map String (LP Inlines)
 inlineCommands = M.fromList $
-  [ ("emph", emph <$> tok)
-  , ("textit", emph <$> tok)
-  , ("textsl", emph <$> tok)
-  , ("textsc", smallcaps <$> tok)
-  , ("sout", strikeout <$> tok)
-  , ("textsuperscript", superscript <$> tok)
-  , ("textsubscript", subscript <$> tok)
+  [ ("emph", extractSpaces emph <$> tok)
+  , ("textit", extractSpaces emph <$> tok)
+  , ("textsl", extractSpaces emph <$> tok)
+  , ("textsc", extractSpaces smallcaps <$> tok)
+  , ("sout", extractSpaces strikeout <$> tok)
+  , ("textsuperscript", extractSpaces superscript <$> tok)
+  , ("textsubscript", extractSpaces subscript <$> tok)
   , ("textbackslash", lit "\\")
   , ("backslash", lit "\\")
   , ("slash", lit "/")
-  , ("textbf", strong <$> tok)
-  , ("textnormal", spanWith ("",["nodecor"],[]) <$> tok)
+  , ("textbf", extractSpaces strong <$> tok)
+  , ("textnormal", extractSpaces (spanWith ("",["nodecor"],[])) <$> tok)
   , ("ldots", lit "…")
   , ("dots", lit "…")
   , ("mdots", lit "…")
   , ("sim", lit "~")
   , ("label", unlessParseRaw >> (inBrackets <$> tok))
   , ("ref", unlessParseRaw >> (inBrackets <$> tok))
+  , ("noindent", unlessParseRaw >> return mempty)
+  , ("textgreek", tok)
+  , ("sep", lit ",")
+  , ("cref", unlessParseRaw >> (inBrackets <$> tok))  -- from cleveref.sty
   , ("(", mathInline $ manyTill anyChar (try $ string "\\)"))
   , ("[", mathDisplay $ manyTill anyChar (try $ string "\\]"))
   , ("ensuremath", mathInline $ braced)
+  , ("texorpdfstring", (\_ x -> x) <$> tok <*> tok)
   , ("P", lit "¶")
   , ("S", lit "§")
   , ("$", lit "$")
@@ -427,15 +458,15 @@ inlineCommands = M.fromList $
   , ("{", lit "{")
   , ("}", lit "}")
   -- old TeX commands
-  , ("em", emph <$> inlines)
-  , ("it", emph <$> inlines)
-  , ("sl", emph <$> inlines)
-  , ("bf", strong <$> inlines)
+  , ("em", extractSpaces emph <$> inlines)
+  , ("it", extractSpaces emph <$> inlines)
+  , ("sl", extractSpaces emph <$> inlines)
+  , ("bf", extractSpaces strong <$> inlines)
   , ("rm", inlines)
-  , ("itshape", emph <$> inlines)
-  , ("slshape", emph <$> inlines)
-  , ("scshape", smallcaps <$> inlines)
-  , ("bfseries", strong <$> inlines)
+  , ("itshape", extractSpaces emph <$> inlines)
+  , ("slshape", extractSpaces emph <$> inlines)
+  , ("scshape", extractSpaces smallcaps <$> inlines)
+  , ("bfseries", extractSpaces strong <$> inlines)
   , ("/", pure mempty) -- italic correction
   , ("aa", lit "å")
   , ("AA", lit "Å")
@@ -494,6 +525,7 @@ inlineCommands = M.fromList $
   , ("citealp", citation "citealp" NormalCitation False)
   , ("citealp*", citation "citealp*" NormalCitation False)
   , ("autocite", citation "autocite" NormalCitation False)
+  , ("smartcite", citation "smartcite" NormalCitation False)
   , ("footcite", inNote <$> citation "footcite" NormalCitation False)
   , ("parencite", citation "parencite" NormalCitation False)
   , ("supercite", citation "supercite" NormalCitation False)
@@ -516,6 +548,7 @@ inlineCommands = M.fromList $
   , ("supercites", citation "supercites" NormalCitation True)
   , ("footcitetexts", inNote <$> citation "footcitetexts" NormalCitation True)
   , ("Autocite", citation "Autocite" NormalCitation False)
+  , ("Smartcite", citation "Smartcite" NormalCitation False)
   , ("Footcite", citation "Footcite" NormalCitation False)
   , ("Parencite", citation "Parencite" NormalCitation False)
   , ("Supercite", citation "Supercite" NormalCitation False)
@@ -542,7 +575,7 @@ inlineCommands = M.fromList $
   ] ++ map ignoreInlines
   -- these commands will be ignored unless --parse-raw is specified,
   -- in which case they will appear as raw latex blocks:
-  [ "noindent", "index" ]
+  [ "index" ]
 
 mkImage :: String -> LP Inlines
 mkImage src = do
@@ -559,7 +592,7 @@ inNote ils =
 
 unescapeURL :: String -> String
 unescapeURL ('\\':x:xs) | isEscapable x = x:unescapeURL xs
-  where isEscapable c = c `elem` "#$%&~_^\\{}"
+  where isEscapable c = c `elem` ("#$%&~_^\\{}" :: String)
 unescapeURL (x:xs) = x:unescapeURL xs
 unescapeURL [] = ""
 
@@ -796,6 +829,14 @@ environment = do
        Just p      -> p <|> rawEnv name
        Nothing     -> rawEnv name
 
+inlineEnvironment :: LP Inlines
+inlineEnvironment = try $ do
+  controlSeq "begin"
+  name <- braced
+  case M.lookup name inlineEnvironments of
+       Just p      -> p
+       Nothing     -> mzero
+
 rawEnv :: String -> LP Blocks
 rawEnv name = do
   let addBegin x = "\\begin{" ++ name ++ "}" ++ x
@@ -807,7 +848,7 @@ rawEnv name = do
 
 ----
 
-type IncludeParser = ParsecT [Char] [String] IO String
+type IncludeParser = ParserT [Char] [String] IO String
 
 -- | Replace "include" commands with file contents.
 handleIncludes :: String -> IO String
@@ -1020,19 +1061,19 @@ environments = M.fromList
   , ("obeylines", parseFromString
                   (para . trimInlines . mconcat <$> many inline) =<<
                   intercalate "\\\\\n" . lines <$> verbEnv "obeylines")
-  , ("displaymath", mathEnv Nothing "displaymath")
-  , ("equation", mathEnv Nothing "equation")
-  , ("equation*", mathEnv Nothing "equation*")
-  , ("gather", mathEnv (Just "gathered") "gather")
-  , ("gather*", mathEnv (Just "gathered") "gather*")
-  , ("multline", mathEnv (Just "gathered") "multline")
-  , ("multline*", mathEnv (Just "gathered") "multline*")
-  , ("eqnarray", mathEnv (Just "aligned") "eqnarray")
-  , ("eqnarray*", mathEnv (Just "aligned") "eqnarray*")
-  , ("align", mathEnv (Just "aligned") "align")
-  , ("align*", mathEnv (Just "aligned") "align*")
-  , ("alignat", mathEnv (Just "aligned") "alignat")
-  , ("alignat*", mathEnv (Just "aligned") "alignat*")
+  , ("displaymath", mathEnv para Nothing "displaymath")
+  , ("equation", mathEnv para Nothing "equation")
+  , ("equation*", mathEnv para Nothing "equation*")
+  , ("gather", mathEnv para (Just "gathered") "gather")
+  , ("gather*", mathEnv para (Just "gathered") "gather*")
+  , ("multline", mathEnv para (Just "gathered") "multline")
+  , ("multline*", mathEnv para (Just "gathered") "multline*")
+  , ("eqnarray", mathEnv para (Just "aligned") "eqnarray")
+  , ("eqnarray*", mathEnv para (Just "aligned") "eqnarray*")
+  , ("align", mathEnv para (Just "aligned") "align")
+  , ("align*", mathEnv para (Just "aligned") "align*")
+  , ("alignat", mathEnv para (Just "aligned") "alignat")
+  , ("alignat*", mathEnv para (Just "aligned") "alignat*")
   ]
 
 letter_contents :: LP Blocks
@@ -1092,8 +1133,8 @@ listenv name p = try $ do
   updateState $ \st -> st{ stateParserContext = oldCtx }
   return res
 
-mathEnv :: Maybe String -> String -> LP Blocks
-mathEnv innerEnv name = para <$> mathDisplay (inner <$> verbEnv name)
+mathEnv :: (Inlines -> a) -> Maybe String -> String -> LP a
+mathEnv f innerEnv name = f <$> mathDisplay (inner <$> verbEnv name)
    where inner x = case innerEnv of
                       Nothing -> x
                       Just y  -> "\\begin{" ++ y ++ "}\n" ++ x ++
@@ -1133,7 +1174,7 @@ paragraph = do
 
 preamble :: LP Blocks
 preamble = mempty <$> manyTill preambleBlock beginDoc
-  where beginDoc = lookAhead $ controlSeq "begin" *> string "{document}"
+  where beginDoc = lookAhead $ try $ controlSeq "begin" *> string "{document}"
         preambleBlock =  (void comment)
                      <|> (void sp)
                      <|> (void blanklines)
@@ -1183,7 +1224,7 @@ citationLabel  = optional sp *>
           <* optional sp
           <* optional (char ',')
           <* optional sp)
-  where isBibtexKeyChar c = isAlphaNum c || c `elem` ".:;?!`'()/*@_+=-[]*"
+  where isBibtexKeyChar c = isAlphaNum c || c `elem` (".:;?!`'()/*@_+=-[]*" :: String)
 
 cites :: CitationMode -> Bool -> LP [Citation]
 cites mode multi = try $ do
@@ -1254,10 +1295,14 @@ parseTableRow :: Int  -- ^ number of columns
 parseTableRow cols = try $ do
   let tableCellInline = notFollowedBy (amp <|> lbreak) >> inline
   let tableCell = (plain . trimInlines . mconcat) <$> many tableCellInline
-  cells' <- sepBy tableCell amp
-  guard $ length cells' == cols
+  cells' <- sepBy1 tableCell amp
+  let numcells = length cells'
+  guard $ numcells <= cols && numcells >= 1
+  guard $ cells' /= [mempty]
+  -- note:  a & b in a three-column table leaves an empty 3rd cell:
+  let cells'' = cells' ++ replicate (cols - numcells) mempty
   spaces
-  return cells'
+  return cells''
 
 simpTable :: LP Blocks
 simpTable = try $ do
@@ -1268,6 +1313,7 @@ simpTable = try $ do
   header' <- option [] $ try (parseTableRow cols <* lbreak <* hline)
   rows <- sepEndBy (parseTableRow cols) (lbreak <* optional hline)
   spaces
+  skipMany (comment *> spaces)
   let header'' = if null header'
                     then replicate cols mempty
                     else header'
