@@ -16,6 +16,13 @@ markdown = readMarkdown def
 markdownSmart :: String -> Pandoc
 markdownSmart = readMarkdown def { readerSmart = True }
 
+markdownCDL :: String -> Pandoc
+markdownCDL = readMarkdown def { readerExtensions = Set.insert
+                 Ext_compact_definition_lists $ readerExtensions def }
+
+markdownGH :: String -> Pandoc
+markdownGH = readMarkdown def { readerExtensions = githubMarkdownExtensions }
+
 infix 4 =:
 (=:) :: ToString c
      => String -> (String, c) -> Test
@@ -140,11 +147,28 @@ tests = [ testGroup "inline code"
           [ "two strongs in emph" =:
              "***a**b **c**d*" =?> para (emph (strong (str "a") <> str "b" <> space
                                          <> strong (str "c") <> str "d"))
+          , "emph and strong emph alternating" =:
+            "*xxx* ***xxx*** xxx\n*xxx* ***xxx*** xxx"
+            =?> para (emph "xxx" <> space <> strong (emph "xxx") <>
+                      space <> "xxx" <> space <>
+                      emph "xxx" <> space <> strong (emph "xxx") <>
+                      space <> "xxx")
+          , "emph with spaced strong" =:
+            "*x **xx** x*"
+            =?> para (emph ("x" <> space <> strong "xx" <> space <> "x"))
+          , "intraword underscore with opening underscore (#1121)" =:
+            "_foot_ball_" =?> para (emph (text "foot_ball"))
           ]
         , testGroup "raw LaTeX"
           [ "in URL" =:
             "\\begin\n" =?> para (text "\\begin")
           ]
+        , testGroup "raw HTML"
+          [ "nesting (issue #1330)" =:
+            "<del>test</del>" =?>
+            rawBlock "html" "<del>" <> plain (str "test") <>
+            rawBlock "html" "</del>"
+          ]
         , "unbalanced brackets" =:
             "[[[[[[[[[[[[[[[hi" =?> para (text "[[[[[[[[[[[[[[[hi")
         , testGroup "backslash escapes"
@@ -163,6 +187,11 @@ tests = [ testGroup "inline code"
           ]
         , testGroup "bare URIs"
           (map testBareLink bareLinkTests)
+        , testGroup "autolinks"
+          [ "with unicode dash following" =:
+            "<http://foo.bar>\8212" =?> para (autolink "http://foo.bar" <>
+                                         str "\8212")
+          ]
         , testGroup "Headers"
           [ "blank line before header" =:
             "\n# Header\n"
@@ -179,17 +208,6 @@ tests = [ testGroup "inline code"
             ("À l'arrivée de la guerre, le thème de l'«impossibilité du socialisme»"
             =?> para "À l’arrivée de la guerre, le thème de l’«impossibilité du socialisme»")
           ]
-        , testGroup "mixed emphasis and strong"
-          [ "emph and strong emph alternating" =:
-            "*xxx* ***xxx*** xxx\n*xxx* ***xxx*** xxx"
-            =?> para (emph "xxx" <> space <> strong (emph "xxx") <>
-                      space <> "xxx" <> space <>
-                      emph "xxx" <> space <> strong (emph "xxx") <>
-                      space <> "xxx")
-          , "emph with spaced strong" =:
-            "*x **xx** x*"
-            =?> para (emph ("x" <> space <> strong "xx" <> space <> "x"))
-          ]
         , testGroup "footnotes"
           [ "indent followed by newline and flush-left text" =:
             "[^1]\n\n[^1]: my note\n\n     \nnot in note\n"
@@ -216,13 +234,59 @@ tests = [ testGroup "inline code"
 --        , testGroup "round trip"
 --          [ property "p_markdown_round_trip" p_markdown_round_trip
 --          ]
+        , testGroup "definition lists"
+          [ "no blank space" =:
+            "foo1\n  :  bar\n\nfoo2\n  : bar2\n  : bar3\n" =?>
+            definitionList [ (text "foo1", [plain (text "bar")])
+                           , (text "foo2", [plain (text "bar2"),
+                                            plain (text "bar3")])
+                           ]
+          , "blank space before first def" =:
+            "foo1\n\n  :  bar\n\nfoo2\n\n  : bar2\n  : bar3\n" =?>
+            definitionList [ (text "foo1", [para (text "bar")])
+                           , (text "foo2", [para (text "bar2"),
+                                            plain (text "bar3")])
+                           ]
+          , "blank space before second def" =:
+            "foo1\n  :  bar\n\nfoo2\n  : bar2\n\n  : bar3\n" =?>
+            definitionList [ (text "foo1", [plain (text "bar")])
+                           , (text "foo2", [plain (text "bar2"),
+                                            para (text "bar3")])
+                           ]
+          , "laziness" =:
+            "foo1\n  :  bar\nbaz\n  : bar2\n" =?>
+            definitionList [ (text "foo1", [plain (text "bar baz"),
+                                            plain (text "bar2")])
+                           ]
+          , "no blank space before first of two paragraphs" =:
+            "foo1\n  : bar\n\n    baz\n" =?>
+            definitionList [ (text "foo1", [para (text "bar") <>
+                                            para (text "baz")])
+                           ]
+          ]
+        , testGroup "+compact_definition_lists"
+          [ test markdownCDL "basic compact list" $
+            "foo1\n:   bar\n    baz\nfoo2\n:   bar2\n" =?>
+            definitionList [ (text "foo1", [plain (text "bar baz")])
+                           , (text "foo2", [plain (text "bar2")])
+                           ]
+          ]
         , testGroup "lists"
           [ "issue #1154" =:
               " -  <div>\n    first div breaks\n    </div>\n\n    <button>if this button exists</button>\n\n    <div>\n    with this div too.\n    </div>\n"
-              =?> bulletList [divWith nullAttr (plain $ text "first div breaks") <>
+              =?> bulletList [divWith nullAttr (para $ text "first div breaks") <>
                               rawBlock "html" "<button>" <>
                               plain (text "if this button exists") <>
-                              rawBlock "html" "</button>\n" <>
-                              divWith nullAttr (plain $ text "with this div too.")]
+                              rawBlock "html" "</button>" <>
+                              divWith nullAttr (para $ text "with this div too.")]
+          , test markdownGH "issue #1636" $
+              unlines [ "* a"
+                      , "* b"
+                      , "* c"
+                      , "    * d" ]
+              =?>
+              bulletList [ plain "a"
+                         , plain "b"
+                         , plain "c" <> bulletList [plain "d"] ]
           ]
         ]
